'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Vue = _interopDefault(require('vue'));

// deprecated
function createFn(fn, args, totalArity) {
  var _this = this;

  var curried = function curried() {
    for (var _len = arguments.length, partialArgs = Array(_len), _key = 0; _key < _len; _key++) {
      partialArgs[_key] = arguments[_key];
    }

    return processInvocation.call(_this, fn, args.concat(partialArgs), totalArity);
  };
  curried.curried = true;
  return curried;
}

function processInvocation(fn, args, totalArity) {
  if (args.length === totalArity) {
    return fn.apply(this, args);
  } else {
    return createFn(fn, args, totalArity);
  }
}

var courier = function courier(count, fn) {
  return createFn(fn, [], count);
};

function isTextNode(node) {
  return node != null && node.text != null && node.isComment === false;
}

var normalizeSlots = function normalizeSlots(slots, context) {
  return Object.keys(slots).reduce(function (arr, key) {
    var template = false;
    slots[key].forEach(function (vnode) {
      if (!vnode.context) {
        if (isTextNode(vnode)) {
          if (!template) {
            slots[key] = context.$createElement('template', { slot: key }, slots[key]);
            template = true;
          }
        } else {
          slots[key].context = context;

          if (!vnode.data) {
            vnode.data = {};
          }
          vnode.data.slot = key;
        }
      }
    });
    return arr.concat(slots[key]);
  }, []);
};

// IE9-11 do not support `Object.assign`
var poly = function poly(target) {
  if (target == null) {
    throw new TypeError('Uncaught TypeError: Cannot convert undefined or null to object');
  }

  for (var i = 0, il = arguments.length <= 1 ? 0 : arguments.length - 1; i < il; i += 1) {
    var source = arguments.length <= i + 1 ? undefined : arguments[i + 1];
    if (source == null) {
      continue;
    }

    for (var key in source) {
      if (Object.hasOwnProperty.call(source, key)) {
        Object.defineProperty(target, key, {
          enumerable: true,
          writable: true,
          value: source[key]
        });
      }
    }
  }

  // $FlowFixMe
  return target;
};

var assign = Object.assign || poly;

var normalizeProps = (function (props) {
  if (!props) {
    return {};
  }
  if (Array.isArray(props)) {
    var obj = {};
    props.forEach(function (key) {
      if (typeof key === 'string') {
        obj[key] = {};
      }
    });
    return obj;
  }
  return assign({}, props);
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var isObject = function isObject(test) {
  return Object.prototype.toString.call(test) === '[object Object]';
};

var justBindOptions = ['listeners', 'nativeOn', 'scopedSlots'];

var justBindFn = function justBindFn(key) {
  return justBindOptions.indexOf(key) > -1;
};

var getOptionsKeys = function getOptionsKeys(options) {
  return Object.keys(options).concat(['listeners', 'props', 'attrs']).filter(function (k, i, a) {
    return a.indexOf(k) === i;
  });
};

var createOptionHandlers = function createOptionHandlers(originalOptions, keys) {
  var options = {};

  keys.forEach(function (key) {
    var option = originalOptions[key];

    if (!option) {
      options[key] = function (owner) {
        return owner;
      };
      return;
    }

    if (typeof option === 'function') {
      options[key] = option;
      return;
    }

    if ((typeof option === 'undefined' ? 'undefined' : _typeof(option)) === 'object' && isObject(option)) {
      var optionKeys = Object.keys(option);

      if (!optionKeys.some(function (k) {
        return typeof option[k] === 'function';
      })) {
        options[key] = function (owner) {
          return assign({}, owner, option);
        };
        return;
      }

      options[key] = function (owner) {
        var _this = this;

        var result = assign({}, owner);
        var justBind = justBindFn(key);

        optionKeys.forEach(function (k) {
          var value = option && option[k];

          if (typeof value === 'function') {
            if (justBind) {
              value = value.bind(_this);
            } else {
              value = value.call(_this, owner);
            }
          }
          result[k] = value;
        });
        return result;
      };
      return;
    }

    options[key] = function () {
      return option;
    };
  });

  return options;
};

var preprocessOptions = function preprocessOptions(originalOptions) {
  var keys = getOptionsKeys(originalOptions);
  var options = createOptionHandlers(originalOptions, keys);

  return function (context, isFunctional) {
    var result = {
      on: {},
      props: {},
      attrs: {}
    };

    keys.forEach(function (key) {
      var owner = isFunctional ? context[key] || context.data[key] : context['$' + key];

      var value = options[key].call(context, owner);

      if (key === 'listeners') {
        key = 'on';
      }

      result[key] = value;
    });

    return result;
  };
};

var getUnusedProps = function getUnusedProps(Component, props) {
  var result = {};
  var target = normalizeProps(typeof Component === 'function' ? Component.options.props : Component.props);

  Object.keys(props).forEach(function (prop) {
    if (target[prop] === undefined) {
      result[prop] = props[prop];
    }
  });

  return result;
};

var createRenderFn = function createRenderFn(Component, options) {
  var getData = preprocessOptions(options || {});

  return function renderHoc(h, context) {
    var data = getData(context || this, !!context);
    var scopedSlots = context && context.data && context.data.scopedSlots || this && this.$scopedSlots;
    var slots = context && context.children || this && this.$slots && normalizeSlots(this.$slots, this.$parent) || [];
    var unusedProps = getUnusedProps(Component, data.props);

    data.scopedSlots = data.scopedSlots || scopedSlots;
    data.attrs = assign({}, unusedProps, data.attrs);

    return h(Component, data, slots);
  };
};

var createRenderFnc = function createRenderFnc(options) {
  var curried = function curried(Component) {
    return createRenderFn(Component, options);
  };
  curried.curried = true;
  return curried;
};

var defaultStrategy = function defaultStrategy(parent, child) {
  return child;
};

var createHOC = function createHOC(Component, options, renderOptions) {
  var hoc = {
    props: normalizeProps(typeof Component === 'function' ? Component.options.props : Component.props),
    mixins: [],
    name: (Component.name || 'Anonymous') + 'HOC',
    render: createRenderFnc(renderOptions)
  };
  if (options) {
    Object.keys(options).forEach(function (key) {
      var child = options && options[key];
      var parent = hoc[key];
      var strategy = Vue.config.optionMergeStrategies[key] || defaultStrategy;

      if (key === 'props') {
        hoc[key] = strategy(parent, normalizeProps(child));
      } else {
        hoc[key] = strategy(parent, child);
      }
    });
  }

  hoc.mixins && hoc.mixins.push({
    created: function created() {
      this.$createElement = this.$parent.$createElement;
    }
  });

  if (hoc.render && hoc.render.curried) {
    hoc.render = hoc.render(Component);
  }

  return hoc;
};

var createHOCc = function createHOCc(options, renderOptions) {
  var curried = function curried(Component) {
    return createHOC(Component, options, renderOptions);
  };
  curried.curried = true;
  return curried;
};

var index = {
  normalizeSlots: normalizeSlots,
  createRenderFn: createRenderFn,
  createRenderFnc: createRenderFnc,
  createHOC: createHOC,
  createHOCc: createHOCc
};

exports.courier = courier;
exports.normalizeSlots = normalizeSlots;
exports.createRenderFn = createRenderFn;
exports.createRenderFnc = createRenderFnc;
exports.createHOC = createHOC;
exports.createHOCc = createHOCc;
exports.default = index;
